<?php

class Model_forum extends Model {

    function __construct() {

		parent::__construct();

    }

    public function addForum($name){
		
		if(!empty($name)) {
			
			try {
				$addUser = $this->db->prepare("INSERT INTO forums (name) VALUES (?)");
                $addUser->execute([$name]);
            }
            catch(PDOException $e){
              die('Ошибка записи: ' . $e->getMessage());
            }
			
		}
		
	}
	
	public function forumInfo($id){
		
		$forumInfo = $this->db->prepare("SELECT * FROM forums WHERE id = ?");
		$forumInfo->execute([$id]);
		$forumInfo->setFetchMode(PDO::FETCH_OBJ);
		return $forumInfo->fetch();
	}
	
	public function partInfo($id){
		
		$partInfo = $this->db->prepare("SELECT * FROM forum_part WHERE id = ?");
		$partInfo->execute([$id]);
		$partInfo->setFetchMode(PDO::FETCH_OBJ);
		return $partInfo->fetch();
	}	
	
	public function issetPart($id){
		
		$issetPart = $this->db->prepare("SELECT * FROM forum_part WHERE id = ?");
		$issetPart->execute([$id]);
		$issetPart->setFetchMode(PDO::FETCH_OBJ);
		
		if($issetPart->rowCount() == 0)
			return false;
		
		$this->part = $issetPart->fetch();
		
		return true;
		
	}
	
	public function issetTheme($curForum, $curPart, $curTheme){
		
		$issetTheme = $this->db->prepare("SELECT * FROM forum_theme WHERE id = ? AND id_forum = ? AND id_part = ?");
		$issetTheme->execute([$curTheme, $curForum, $curPart]);
		$issetTheme->setFetchMode(PDO::FETCH_OBJ);
		
		if($issetTheme->rowCount() == 0)
			return false;
		
		$this->theme = $issetTheme->fetch();
		
		return true;
		
	}
	
    public function addForumPart($id, $name){
		
		if(!empty($id) && !empty($name)) {
			
			try {
				$addPart = $this->db->prepare("INSERT INTO forum_part (id_forum,name) VALUES (?,?)");
                $addPart->execute([$id, $name]);
            }
            catch(PDOException $e){
              die('Ошибка записи: ' . $e->getMessage());
            }
			
		}
		
	}

    public function addTheme($id_forum, $id_part, $id_author, $theme_name, $theme_message){
			
			try {
				$addTheme = $this->db->prepare("INSERT INTO forum_theme (id_forum,id_part,id_author,theme_name,time) VALUES (?,?,?,?,?)");
                $addTheme->execute([$id_forum, $id_part, $id_author, $theme_name, time()]);	
				$this->id_theme = $this->db->lastInsertId();
				
				$addMessage = $this->db->prepare("INSERT INTO forum_message (id_forum,id_part,id_theme,id_author,message,time) VALUES (?,?,?,?,?,?)");
                $addMessage->execute([$id_forum, $id_part, $this->id_theme, $id_author, $theme_message, time()]);
				
            }
            catch(PDOException $e){
              die('Ошибка записи: ' . $e->getMessage());
            }
		
	}

    public function addMessage($id_forum, $id_part, $id_theme, $id_author, $message){
			
			try {
				$addMessage = $this->db->prepare("INSERT INTO forum_message (id_forum,id_part,id_theme,id_author,message,time) VALUES (?,?,?,?,?,?)");
                $addMessage->execute([$id_forum, $id_part, $id_theme, $id_author, $message, time()]);
            }
            catch(PDOException $e){
              die('Ошибка записи: ' . $e->getMessage());
            }
		
	}	
	
	public function readForums(){
		
		$result = [];
		
		$forums = $this->db->query("SELECT * FROM forums");
        $forums->setFetchMode(PDO::FETCH_OBJ);
		
         while($forum = $forums->fetch())
         {
            $forum->name = htmlspecialchars($forum->name);
            $result[] = $forum;
         }
		
		return $result;
		
	}
	
	public function readForumsParts(){
		
		$resultForum = [];
		$resultPart = [];
		
		$forums = $this->db->query("SELECT `forums`.*, (SELECT COUNT(`forum_part`.`id`) FROM `forum_part` WHERE `forum_part`.`id_forum`=`forums`.`id`) AS count FROM `forums`");
        $forums->setFetchMode(PDO::FETCH_OBJ);
		
         while($forum = $forums->fetch())
         {
            $forum->name = htmlspecialchars($forum->name);
            $resultForum[] = $forum;
			
			$parts = $this->db->prepare("SELECT `forum_part`.*, (SELECT COUNT(`forum_theme`.`id`) FROM `forum_theme` WHERE `forum_theme`.`id_part`=`forum_part`.`id`) AS count FROM forum_part WHERE id_forum = ?");
			$parts->execute([$forum->id]);
			
			$parts->setFetchMode(PDO::FETCH_OBJ);
					
			while($part = $parts->fetch()){
				
				$part->name = htmlspecialchars($part->name);
				$resultPart[] = $part;
			
			}
			
         }
		
		return [$resultForum, $resultPart];
		
	}

	public function readThemes($part, $start){
		
		$go = ($start * 10) - 10;
		
		$result = [];
		
		$themes = $this->db->prepare("SELECT `forum_theme`.*, (SELECT COUNT(`forum_message`.`id`) FROM `forum_message` WHERE `forum_message`.`id_theme`=`forum_theme`.`id`) AS count FROM forum_theme WHERE id_part = ? ORDER BY id DESC LIMIT $go ,10 ");
		
		
		$themes->execute([$part]);
		
		$themes->setFetchMode(PDO::FETCH_OBJ);
		
		while($theme = $themes->fetch()){
			
			$theme->theme_name = htmlspecialchars($theme->theme_name);
			$result[] = $theme; 
		}
		
		return $result;
	}

	public function readMessages($theme, $start){
		
		$pagination = 10;
		
		$go = ($start * $pagination) - $pagination;
		
		$result = [];
		
		$login = new User();
		
		
		$messages = $this->db->prepare("SELECT * FROM forum_message WHERE id_theme = ? ORDER BY id ASC LIMIT $go ,10 ");
		$messages->execute([$theme]);
		
		$messages->setFetchMode(PDO::FETCH_OBJ);
		
		while($message = $messages->fetch()){
			
			$message->id_author = $login->login($message->id_author);
			$message->message = htmlspecialchars($message->message);
			$result[] = $message; 
		}
		
		return $result;
	}	
	
	public function countForums(){
		
		$count = $this->db->prepare("SELECT COUNT(*) as count FROM forums");
		$count->execute();
		return $count->fetch(PDO::FETCH_OBJ)->count;
		
		
	}
	
	public function read($start = 1){
		
		$go = ($start * 10) - 10;
		
		
		$result = [];
		
		$login = new User();
		
		$messages = $this->db->query("SELECT * FROM guestbook ORDER BY id DESC LIMIT $go ,10 ");
        $messages->setFetchMode(PDO::FETCH_OBJ);
		
         while($message = $messages->fetch())
         {
			$message->id_user = $login->login($message->id_user);
            $message->message = htmlspecialchars($message->message);
			$message->time = date('d-m-y h:i:s', $message->time);
			
            $result[] = $message;
         }
		
		return $result;
		
	}
	
	public function deleteForum($id){
		
		$deleteForum = $this->db->prepare("DELETE FROM forums WHERE id = ? ");
		$deleteForum->execute([$id]);
		
		$deletePart = $this->db->prepare("DELETE FROM forum_part WHERE id_forum = ? ");
		$deletePart->execute([$id]);
		
		$deleteTheme = $this->db->prepare("DELETE FROM forum_theme WHERE id_forum = ? ");
		$deleteTheme->execute([$id]);
		
		$deleteMessage = $this->db->prepare("DELETE FROM forum_message WHERE id_forum = ? ");
		$deleteMessage->execute([$id]);
		
	}
	
	public function deletePart($id){
		
		$deletePart = $this->db->prepare("DELETE FROM forum_part WHERE id = ? ");
		$deletePart->execute([$id]);
		
		$deleteTheme = $this->db->prepare("DELETE FROM forum_theme WHERE id_part = ? ");
		$deleteTheme->execute([$id]);
		
		$deleteMessage = $this->db->prepare("DELETE FROM forum_message WHERE id_part = ? ");
		$deleteMessage->execute([$id]);
		
	}	
	
	public function navigation_themes($id_part){
		
		$result = [];
		
		$themes = $this->db->prepare("SELECT COUNT(*) as count FROM forum_theme WHERE id_part = ?");
		$themes->execute([$id_part]);
		
		$count = $themes->fetch(PDO::FETCH_OBJ)->count;
		
		$pages = ceil($count/10);
		
		if($pages > 1)
            for($i=1; $i<=$pages; ++$i) 
				$result[] = $i;
				
	return $result;
	}	

	public function navigation_messages($id_theme){
		
		$result = [];
		
		$messages = $this->db->prepare("SELECT COUNT(*) as count FROM forum_message WHERE id_theme = ?");
		$messages->bindValue(1, $id_theme, PDO::PARAM_STR);
		$messages->execute();
		
		$count = $messages->fetch(PDO::FETCH_OBJ)->count;
		
		$pages = ceil($count/10);
		
		//if($pages > 1)
            for($i=1; $i<=$pages; ++$i) 
				$result[] = $i;
				
	return $result;
	}		
	
}
